@@ -139,11 +139,17 @@ csinkrend_onnewdata_cb (CSink * sink)
 static void
 csinkrend_onerror_cb (CSink * sink)
 {
-    ENode *node;
+    ENode *node = NULL;
+    EBuf *remote_ip_address;
     gchar *function;
 
     EDEBUG (("csinkrend", "some csink received an error."));
 
+    /* Make the remote ip into dotted quads and put it in an EBuf */
+    remote_ip_address = ebuf_new_with_str
+        (inet_ntoa ( *(struct in_addr*) &(CSINK_INET (sink)->ip)) );
+    enode_attrib (node, "__remote-ip", remote_ip_address);
+
     node = (ENode *) csink_get_user_data (sink);
     EDEBUG (("csinkrend", "sink = %p, node = %p", sink, node));
     function = enode_attrib_str (node, "onerror", NULL);
@@ -624,7 +630,7 @@ register_inet_attribs (Element *element)
   e_attr->description =
     "Action wanted to perform (ie \"connect\", \"disconnect\".)";
   e_attr->value_desc = "choice";
-  e_attr->possible_values = "connect,close,listen";
+  e_attr->possible_values = "connect,disconnect,listen";
   e_attr->set_attr_func = csinkrend_action_attr_set;
   element_register_attrib (element, e_attr);
 
@@ -643,7 +649,9 @@ void
 rendcsink_init (int flags)
 {
     Element *element;
+#ifdef HAVE_OPENSSL
     ElementAttr *e_attr;
+#endif
 
     /* SIGPIPE may be sent to us when we try to write down a closed
      * socket. Lets ignore it. */
@@ -682,7 +690,6 @@ rendcsink_init (int flags)
 
     register_inet_attribs (element); /* define it's base attributes */
 
-
     e_attr = g_new0 (ElementAttr, 1); /* override the action attribute */
     e_attr->attribute = "action";
     e_attr->description =
