@@ -28,8 +28,11 @@ void csink_ssl_init (CSinkSSL * sink);
 /* Internal functions. */
 static int csink_ssl_verify_callback (int ok, X509_STORE_CTX *ctx);
 
-#define SSL_MAX_WRITE_CHUNKS 3
 
+static GHashTable * csink_certs = NULL;
+
+
+#define SSL_MAX_WRITE_CHUNKS 3
 
 static int csink_ssl_seed_random (void)
 {
@@ -110,6 +113,11 @@ void csink_ssl_init (CSinkSSL * sink)
     CSINK_SOCKET (sink)->can_read_action = 
 	(CSinkSocketCanReadFunc) csink_ssl_can_read_action;
 
+    /* Make a hash to lookup the csink associated with an ssl cert. */
+    if (NULL == csink_certs)
+    {
+        csink_certs = g_hash_table_new (g_direct_hash, g_direct_equal);
+    }
 
     if (0 == csink_ssl_seed_random ())		/* Unable to seed. */
         csink_on_error (CSINK (sink), "SSL_NOSEED");
@@ -127,13 +135,11 @@ static void
 csink_ssl_init_check (void)
 {
     if (FALSE == ssl_is_initialized) {
-        /* Ensure that openssl has bee initialized. */
+        /* Ensure that openssl has been initialized. */
 	CDEBUG (("csinkssl", "initing ssl libraries."));
 	SSL_library_init ();
         SSL_load_error_strings ();
 	ERR_load_crypto_strings ();
-
-        /* SSL_library_init (); */      
     }
     
     ssl_is_initialized = TRUE;
@@ -147,8 +153,8 @@ CSinkSSL * csink_ssl_create (CSinkSSL * old_sink)
 
     CDEBUG (("csinkssl", "in csink_ssl_create"));
 
-    /* make sure the ssl libraries are initialized before we go ahead and try
-       using them */
+    /* Make sure the ssl libraries are initialized before we go ahead and try
+       using them, this inits if needed. */
     csink_ssl_init_check ();
 
     sink = g_new0 (CSinkSSL, 1);
@@ -159,8 +165,12 @@ CSinkSSL * csink_ssl_create (CSinkSSL * old_sink)
         /* Set our SSL params. */
         sink->meth = SSLv3_method ();
         sink->ctx = SSL_CTX_new (sink->meth);
+
         SSL_CTX_set_options (sink->ctx, SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
+	/* Make a cert to csink mapping. */
+	g_hash_table_insert (csink_certs, (void *)sink->ctx->cert_store, sink);
+
         /* From s_server.c: "anything will do". */
         sink->session_id_context = local_context++;
 
@@ -176,19 +186,21 @@ CSinkSSL * csink_ssl_create (CSinkSSL * old_sink)
         /* Clone out of the old sink. */
         sink->meth = old_sink->meth;
         sink->ctx = old_sink->ctx;
+
+	/* Make a cert to csink mapping. */
+	CDEBUG (("csinkssl", "csink %p has cert %p", sink,
+				sink->ctx->cert_store));
+	g_hash_table_insert (csink_certs, (void *)sink->ctx->cert_store, sink);
+
         sink->verify_depth = old_sink->verify_depth;
+
+	csink_set_new_data_func (CSINK (sink), CSINK (old_sink)->on_new_data);
+	csink_set_close_func    (CSINK (sink), CSINK (old_sink)->on_close);
+	csink_set_connect_func  (CSINK (sink), CSINK (old_sink)->on_connect);
+	csink_set_error_func    (CSINK (sink), CSINK (old_sink)->on_error);
+	csink_set_user_data     (CSINK (sink), CSINK (old_sink)->user_data);
     }
 
-    if (NULL == old_sink)
-        return sink;
-
-    /* Need to clone stuff out of old_sink. */
-    csink_set_new_data_func (CSINK (sink), CSINK (old_sink)->on_new_data);
-    csink_set_close_func    (CSINK (sink), CSINK (old_sink)->on_close);
-    csink_set_connect_func  (CSINK (sink), CSINK (old_sink)->on_connect);
-    csink_set_error_func    (CSINK (sink), CSINK (old_sink)->on_error);
-    csink_set_user_data     (CSINK (sink), CSINK (old_sink)->user_data);
-
     return sink;
 }
 
@@ -226,7 +238,6 @@ void csink_ssl_set_certdir (CSinkSSL * sink, char * cert_dir)
 }
 
 
-
 int csink_ssl_cert_info (CSinkSSL * sink)
 {
     int ret;
@@ -285,10 +296,8 @@ int csink_ssl_cert_info (CSinkSSL * sink)
 
     /* Setup the verification scheme.  
      * Feature request: Let the user choose to do 'SSL_VERIFY_CLIENT_ONCE'. */
-    SSL_CTX_set_verify (sink->ctx, 
-		SSL_VERIFY_PEER 		| 
-		SSL_VERIFY_CLIENT_ONCE, 
-		csink_ssl_verify_callback);
+    SSL_CTX_set_verify (sink->ctx, SSL_VERIFY_PEER | SSL_VERIFY_CLIENT_ONCE, 
+			csink_ssl_verify_callback);
 
 
     CDEBUG (("csinkssl", "cert paths are alright."));
@@ -296,9 +305,8 @@ int csink_ssl_cert_info (CSinkSSL * sink)
 }
 
 
-/* We got the socket connection up, now we need to 
- * connect up the ssl protocol.  If we can't 
- * complete this time then we call ourselves again.
+/* We got the socket connection up, now we need to connect up the ssl protocol.
+ * If we can't  complete this time then we call ourselves again.
  */
 static void csink_ssl_open_action (CSink * sink_)
 {
@@ -333,6 +341,8 @@ static void csink_ssl_open_action (CSink * sink_)
 
     if (0 == res) {
 	CDEBUG (("csinkssl", "open_action: handshake failure"));
+	CDEBUG (("csinkssl", "SSL says: %s",
+		 ERR_error_string (err, NULL)));
 	csink_on_error (CSINK (sink), "SSL_HANDSHAKE");
 	csink_close (CSINK (sink));
 	return;
@@ -372,9 +382,9 @@ static void csink_ssl_open_action (CSink * sink_)
         /* The ssl is connected up. */
         sink->status = SSS_SSL_CONNECTED;
 
-        if (CSINK_SSL_TYPE == CSINK (sink)->csink_type) {
+        if (CSINK_TYPE_CHECK (sink, CSINK_SSL_TYPE)) {
             CDEBUG (("csinkssl", "can write now..."));
-            CSINK (sink)->flags = CSSF_CAN_WRITE;
+            CSINK (sink)->flags = CSSF_CONNECTED;
         }
         
         /* We are ready to rock and roll. */
@@ -422,23 +432,33 @@ static void csink_ssl_open_action (CSink * sink_)
 
             break;
         case SSL_ERROR_SYSCALL:
-
-            if (EAGAIN != errno || EINTR != errno) {
-                csink_on_error (CSINK(sink), "UNKNOWN");
+	    /* A fun nested switch to handle errno problems. */
+	    switch (errno) {
+	    case EAGAIN:	/* Just retry on these ones. */
+	    case EINTR:
+		/* Resetup the io watcher. */
+		if (CSINK_SOCKET (sink)->read_watch_tag) {
+		    csink_remove_fd (CSINK_SOCKET (sink)->read_watch_tag,
+			"Adding read/write/err to ssl read tag");
+		}
+		CSINK_SOCKET (sink)->read_watch_tag =
+			csink_add_fd (CSINK_SOCKET (sink)->fd,
+				EIO_WRITE | EIO_READ | EIO_ERROR,
+				csink_ssl_open_action, CSINK(sink),
+				"SSL Socket Read Tag (read/write/err added in)");
+		break;
+	    case ECONNRESET:
+		csink_on_error (CSINK(sink), "ECONNRESET");
+		csink_close (CSINK(sink));
+		break;
+	    case EPIPE:
+		csink_on_error (CSINK(sink), "SEND_PIPE");
+		csink_close (CSINK(sink));
+		break;
+            default:
+		csink_on_error (CSINK(sink), "UNKNOWN");
                 csink_close (CSINK (sink));
-            } else {
-               /* Resetup the io watcher. */
-               if (CSINK_SOCKET (sink)->read_watch_tag)
-                   csink_remove_fd (CSINK_SOCKET (sink)->read_watch_tag,
-                   "Adding read/write/err to ssl read tag");
-               CSINK_SOCKET (sink)->read_watch_tag =
-                   csink_add_fd (CSINK_SOCKET (sink)->fd,
-                              EIO_WRITE | EIO_READ | EIO_ERROR,
-                              csink_ssl_open_action, CSINK(sink),
-                              "SSL Socket Read Tag (read/write/err added in)");
-            }
-            break;
-
+	    }
 	default:
             CDEBUG (("csinkssl", "protocol error"));
             /* Big trouble, protocol error. */
@@ -462,7 +482,11 @@ static void csink_ssl_open (CSinkSSL * sink)
      * Simple. 
      */
 
-    CDEBUG (("csinkssl", "in csink_ssl_open"));
+    CDEBUG (("csinkssl", "csink_ssl_open status: %i", sink->status));
+
+    if (CSINK_TYPE_CHECK (sink, CSINK_SSL_TYPE)) {
+        CSINK (sink)->flags = CSSF_CONNECTING;    
+    }
 
     if (CSINK_INET(sink)->socket.status & SOCKET_INET_DNS_INPROGRESS) {
 	CDEBUG (("csinkssl", "open, dealing with DNS lookup..\n"));
@@ -470,18 +494,16 @@ static void csink_ssl_open (CSinkSSL * sink)
 	    CSINK_INET(sink)->socket.status |= SOCKET_CONNECT_INPROGRESS;
 	    csink_inet_set_on_dns_lookup_success
 		(CSINK_INET(sink), (CSinkCallbackFunc)csink_ssl_open);
-	    CDEBUG (("csinkinet",
+	    CDEBUG (("csinkssl",
 		     "open, queued connect after lookup completes..\n"));
 	}
 	return;
     }
 
-
     sink->ssl = SSL_new (sink->ctx);
     SSL_clear (sink->ssl);
 
     sink->status = SSS_SSL_WAITING;
-    CSINK (sink)->flags = CSSF_TRYING;    
 
     CDEBUG (("csinkssl", "open, calling socket open to finish up"));
     csink_socket_open (CSINK_SOCKET (sink));
@@ -530,11 +552,13 @@ csink_ssl_accept_action (CSink * sink_)
 	/* do the inet level accept */
 	sink = CSINK_SSL (csink_inet_do_accept (CSINK_INET (oldsink)));
 	if (NULL == sink) {
-	    CDEBUG (("csinkssl", "accept_action, failed inet accept, returning."));
+	    CDEBUG (("csinkssl",
+		"csink_ssl_accept_action, failed inet accept, returning."));
 	    return;
 	}
 	
-	CDEBUG (("csinkssl", "accept_action, done inet accept, on to ssl accept."));
+	CDEBUG (("csinkssl",
+		"accept_action, done inet accept, on to ssl accept."));
 
 	sink->ssl = SSL_new (sink->ctx);
 	SSL_set_fd (sink->ssl, CSINK_SOCKET (sink)->fd);
@@ -542,7 +566,8 @@ csink_ssl_accept_action (CSink * sink_)
     } else {
 	/* otherwise we know we've already made this and are passing it back to
 	   ourselves. */
-	CDEBUG (("csinkssl", "accept_action, back again, no inet setup needed."));
+	CDEBUG (("csinkssl",
+			"accept_action, back again, no inet setup needed."));
 	sink = oldsink;
     }
 
@@ -590,12 +615,13 @@ csink_ssl_accept_action (CSink * sink_)
                         csink_ssl_can_read_action, CSINK(sink),
 			"SSL Accept()ed Sink Tag");
 
-        CDEBUG (("csinkssl", "calling connect"));
+        CDEBUG (("csinkssl", "calling on_connect"));
 
         /* The ssl is connected up. */
         sink->status = SSS_SSL_CONNECTED;
-        if (CSINK_SSL_TYPE == CSINK (sink)->csink_type) {
-            CSINK (sink)->flags = CSSF_CAN_WRITE;
+
+        if (CSINK_TYPE_CHECK (sink, CSINK_SSL_TYPE)) {
+            CSINK (sink)->flags = CSSF_CONNECTED;
         }
 
         /* Call the user on connect function. */
@@ -618,26 +644,40 @@ csink_ssl_accept_action (CSink * sink_)
 
     switch (err) {
     case SSL_ERROR_WANT_READ:
-    case SSL_ERROR_WANT_WRITE:	
-	CDEBUG (("csinkssl", "csink_ssl_accept: received MUST_(read/write)."));
-	
-	/* Resetup the io watcher to wait for what we need to complete
-	   this SSL_accept */
+	CDEBUG (("csinkssl", "csink_ssl_accept: received WANT_READ."));
+
+	/* Resetup the io watcher to call when there's more to be read. */
 	if (CSINK_SOCKET (sink)->read_watch_tag) {
 	    csink_remove_fd (CSINK_SOCKET (sink)->read_watch_tag,
-			     "Adding read/write/err for SSL accept");
+			     "Adding read/err for SSL accept");
+	}
+
+        CSINK_SOCKET (sink)->read_watch_tag =
+		csink_add_fd (CSINK_SOCKET (sink)->fd,
+			EIO_READ | EIO_ERROR,
+                          csink_ssl_accept_action,
+                          CSINK(sink),
+                          "SSL Accept tag (read/error added) ");
+        return;
+    case SSL_ERROR_WANT_WRITE:	
+	CDEBUG (("csinkssl", "csink_ssl_accept: received WANT_WRITE."));
+	
+	/* Resetup the io watcher to finish up our part of the connection. */
+	if (CSINK_SOCKET (sink)->read_watch_tag) {
+	    csink_remove_fd (CSINK_SOCKET (sink)->read_watch_tag,
+			     "Adding write for SSL accept");
 	}	
 	
 	CSINK_SOCKET (sink)->read_watch_tag =
-	    csink_add_fd (CSINK_SOCKET (sink)->fd,
-			  EIO_READ | EIO_WRITE | EIO_ERROR,
-			  csink_ssl_accept_action, 
-			  CSINK(sink),
-			  "SSL Accept tag (read/write/error added) ");
+		csink_add_fd (CSINK_SOCKET (sink)->fd,
+			EIO_WRITE,
+			csink_ssl_accept_action, 
+			CSINK(sink),
+			"SSL Accept tag (write added) ");
 	return;
     default:
-	/* Big trouble, protocol error. */
-	CDEBUG (("csinkssl", "protocol error"));
+	/* Big trouble, protocol error.  Let's see if we can find the cause. */
+	CDEBUG (("csinkssl", "protocol error %i", err));
 	csink_on_error (CSINK(sink), "SSL_PROTOCOL");
 	csink_close (CSINK (sink));
 	return;
@@ -649,19 +689,21 @@ csink_ssl_accept_action (CSink * sink_)
 static void
 csink_ssl_close (CSinkSSL * sink)
 {
-
     CDEBUG (("csinkssl", "in csink_ssl_close"));
 
-
     /* Stop the SSL session. */
-    SSL_shutdown (sink->ssl); 
-    SSL_free (sink->ssl);
-
+    if (sink->ssl) {
+	SSL_shutdown (sink->ssl); 
+	SSL_free (sink->ssl);
+    }
     sink->status = SSS_NOTCONNECTED;
+    g_hash_table_remove (csink_certs, sink);
 
     csink_inet_close (CSINK_INET (sink));
 
-    CSINK (sink)->flags = CSSF_CLOSED;
+    if (CSINK_TYPE_CHECK (sink, CSINK_SSL_TYPE)) {
+        CSINK (sink)->flags = CSSF_CLOSED;
+    }
 }
 
 
@@ -669,15 +711,15 @@ csink_ssl_close (CSinkSSL * sink)
 static void
 csink_ssl_free (CSinkSSL * sink)
 {
-  csink_inet_release (CSINK_INET(sink));
+    csink_inet_release (CSINK_INET(sink));
 
-  free (sink);
+    free (sink);
 }
 
 static gint
 csink_ssl_write (CSinkSSL * sink, CBuf * message)
 {
-    CDEBUG (("csinkssl", "in csink_ssl_write"));
+    CDEBUG (("csinkssl", "in csink_ssl_write, fd: %i", CSINK_SOCKET(sink)->fd));
 
     if (FALSE == csink_write_sanity (CSINK(sink), message) ) {
         return FALSE;
@@ -685,6 +727,34 @@ csink_ssl_write (CSinkSSL * sink, CBuf * message)
 
     CDEBUG (("csinkssl", "...sane..."));
 
+    switch (sink->status)
+      {
+        case SSS_CONNECTED:
+	  CDEBUG (("csinkssl", "status: SSS_CONNECTED"));
+	  break;
+        case SSS_NOTCONNECTED:
+	  CDEBUG (("csinkssl", "status: SSS_NOTCONNECTED"));
+	  break;
+        case SSS_WAITING:
+	  CDEBUG (("csinkssl", "status: SSS_WAITING"));
+	  break;
+        case SSS_SSL_WAITING:
+	  CDEBUG (("csinkssl", "status: SSS_SSL_WAITING"));
+	  break;
+        case SSS_SSL_CONNECTED:
+	  CDEBUG (("csinkssl", "status: SSS_SSL_CONNECTED"));
+	  break;
+        case SSS_SSL_CONNECTING:
+	  CDEBUG (("csinkssl", "status: SSS_SSL_CONNECTING"));
+	  break;
+        case SSS_SSL_ACCEPTING:
+	  CDEBUG (("csinkssl", "status: SSS_SSL_ACCEPTING"));
+	  break;
+	default:
+	  CDEBUG (("csinkssl", "status: unknown"));
+	  break;
+      }
+
     if (SSS_SSL_CONNECTED != sink->status) {
         /* Can't queue message. */
         csink_on_error (CSINK (sink), "DISCONN_WRITE");
@@ -803,13 +873,12 @@ csink_ssl_can_read_action (CSink *sink_)
 	  break;
 
       case SSL_ERROR_SYSCALL:
-	  CDEBUG (("csinkssl", "read_action, SSL_ERROR_SYSCALL"));
-	  if (res == 0) {
-	      csink_on_error (CSINK (sink), "SSL_PROTOCOL");
-	      csink_close (CSINK (sink));
-	      return;
-	  }
+	  CDEBUG (("csinkssl", "read_action, SSL_ERROR_SYSCALL, %s (%i)",
+				  strerror (errno), errno));
 	  switch (errno) {
+	  case 0:		/* Just a normal disconnect. */
+	      csink_close (CSINK (sink));
+	      break;
 	  case EPIPE:		/* Connection broken. */
 	      csink_on_error (CSINK (sink), "SEND_PIPE");
 	      csink_close (CSINK (sink));
@@ -853,8 +922,8 @@ csink_ssl_can_read_action (CSink *sink_)
 	  csink_close (CSINK (sink));
 	  break;
 
-      /* from here on, i'm taking these errors right out of the openssl sources, */
-      /* not from the docs. */
+      /* from here on, i'm taking these errors right out of the openssl
+       * sources, not from the docs. */
 
       case SSL_ERROR_WANT_CONNECT:
 	  CDEBUG (("csinkssl", "read_action, got SSL_ERROR_WANT_CONNECT"));
@@ -907,7 +976,7 @@ static void csink_ssl_can_write_action (CSink * sink_)
 
     CDEBUG (("csinkssl", "in csink_ssl_can_write_action"));
 
-    /* ensure that we are supposed to be here: */
+    /* Ensure that we are supposed to be here: */
     if (SSS_SSL_CONNECTED != sink->status) {
         /* Need to clean up stuff, there is user error going on
          * if this is the case. We need to call the cleanup code.
@@ -1020,6 +1089,10 @@ static void csink_ssl_can_write_action (CSink * sink_)
 		csink_on_error (CSINK (sink), "ENOTSOCK");
 		csink_close (CSINK (sink));
 		break;
+	    case ECONNRESET:
+		csink_on_error (CSINK (sink), "ECONNRESET");
+		csink_close (CSINK (sink));
+		break;
 	    case EMSGSIZE:
 		csink_on_error (CSINK (sink), "EMSGSIZE");
 		csink_close (CSINK (sink));
@@ -1050,9 +1123,8 @@ static void csink_ssl_can_write_action (CSink * sink_)
 	    csink_close (CSINK (sink));
 	    break;
 
-        /* from here on, i'm taking these errors right out of the openssl sources, */
-        /* not from the docs. */
-
+        /* From here on, I'm taking these errors right out of the openssl
+	 * sources, not from the docs. */
 	case SSL_ERROR_WANT_CONNECT:
 	    CDEBUG (("csinkssl", "read_action, got SSL_ERROR_WANT_CONNECT"));
 	    csink_on_error (CSINK (sink), "UNKNOWN");
@@ -1068,7 +1140,7 @@ static void csink_ssl_can_write_action (CSink * sink_)
 	    break;
 	}
 
-	/* save back what wasn't written and return. */
+	/* Save back what wasn't written and return. */
 	partial = cbuf_new_with_data (cur->str+written, 
 				      cur->len-written);
 	g_ptr_array_add_at_start (CSINK(sink)->outQueue,
@@ -1079,16 +1151,15 @@ static void csink_ssl_can_write_action (CSink * sink_)
 	return;
     }
 
-    /* don't disconnect write tag if we are not done writing. */
+    /* Don't disconnect write tag if we are not done writing. */
     if (CSINK (sink)->outQueue->len != 0)
 	return;
 
     CDEBUG (("csinkssl", "Done sending, disconnecting write tag."));
     
-    /* Disconnect write tag now that everthing is done being
-     * sent. */
-    /* note that it might not have been set in the first place if we were
-       dispatched here by must_write */
+    /* Disconnect write tag now that everthing is done being sent. */
+    /* Note that it might not have been set in the first place if we were
+       dispatched here by must_write. */
     if (CSINK_SOCKET (sink)->write_watch_tag) {
 	csink_remove_fd (CSINK_SOCKET (sink)->write_watch_tag,
 	  "SSL write Queue empty");
@@ -1097,33 +1168,43 @@ static void csink_ssl_can_write_action (CSink * sink_)
 }
 
 
-/* The ssl callback used to verify the new connection's
- * cert. */
-static int csink_ssl_verify_callback (int ok,
-				X509_STORE_CTX *ctx)
+/* The ssl callback used to verify the new connection's cert. */
+static int csink_ssl_verify_callback (int ok, X509_STORE_CTX *ctx)
 {
-  CDEBUG (("csinkssl", "in csink_ssl_verify_callback"));
+    CSink * sink;
 
-  if (0 == ok) {
-      /* Verify error. */
-      /* FIXME: The certificate might be farther away from the Authority that
-       * allowed by default, but this sink should be able to decide. */
-  }
+    CDEBUG (("csinkssl", "in csink_ssl_verify_callback"));
 
+    sink = g_hash_table_lookup (csink_certs, ctx->ctx);
+    CDEBUG (("csinkssl", "sink %p, from ctx->ctx %p, ctx %p",
+			    sink, ctx->ctx, ctx));
+
+    if (0 == ok) {
+	/* Verify error. */
+	/* FIXME: The certificate might be farther away from the Authority that
+	 * allowed by default, but this sink should be able to decide. */
+    }
+
+    CDEBUG (("csinkssl", "in csink_ssl_verify_callback, error code %i",
+			    ctx->error));
+    switch (ctx->error) {
+    case 0:		/* No error. */
+	break;
+    case X509_V_ERR_CERT_HAS_EXPIRED:
+	csink_on_error (sink, "SSL_EXPIRED_CERT");
+	break;
+    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT_LOCALLY:
+	csink_on_error (sink, "SSL_HASH_MISSING");
+	break;
+    case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
+        /* FIXME: its possible that we want to accept certs even if they
+	 * expired.  Verisign is a racket. */
+        break;
+    default:		/* Everything else. */
+	csink_on_error (sink, "UNKNOWN");
+        break;
+    }
   
-  switch (ctx->error) {
-  case X509_V_ERR_CERT_HAS_EXPIRED:
-  case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
-      /* FIXME: its possible that we want to accept certs even if they expired.
-       * Verisign is a racket. */
-      break;
-
-  default:
-      /* Everything else. */
-      break;
-  }
-  
-  return (ok);		/* Pass the error on. */
+    return (ok);		/* Pass the error on. */
 }
 
-
