@@ -515,6 +515,27 @@ rendgtk_ctree_row_collapse_callback (GtkWidget * ctree_widget,
 	enode_call_ignore_return (ctree, function, "n", node);
 }
 
+static void
+rendgtk_ctree_column_clicked_callback (GtkWidget *ctree_widget, gint column,
+                                       gpointer user_data)
+{
+    ENode *node;
+    ENode *ctree = user_data;
+    gchar *function;
+    GSList *children;
+
+    children = enode_children (ctree, "ctree-column");
+    node = g_slist_nth_data (children, column);
+
+    g_slist_free (children);
+    if (!node) {
+        return;
+    }
+ 
+    function = enode_attrib_str (node, "onclick", NULL);
+    if (function)
+	enode_call_ignore_return (ctree, function, "n", node);
+}
 
 static gint
 rendgtk_ctree_row_expanded_attr_set (ENode * node, EBuf * attr, EBuf * value)
@@ -816,9 +837,18 @@ rendgtk_ctree_columns_sync (ENode * ctree_node)
     tmp = children;
     while (tmp) {
 	ENode *node = tmp->data;
+        char *oldtitle;
+        EBuf *title;
 
-	gchar *title = enode_attrib_str (node, "title", NULL);
-	gtk_clist_set_column_title (GTK_CLIST (ctree), column, title);
+        oldtitle = gtk_clist_get_column_title (GTK_CLIST (ctree), column);
+	title = enode_attrib (node, "title", NULL);
+
+        /* If they aren't using 'title' attribute, and are packing widgets
+         * into the column, we can't set title at all.  So, we use the old
+         * title to see if it was previously set.  In this case, we want
+         * to be able to see if it's unset, or simply being set to "" */
+        if (ebuf_not_empty (title) || oldtitle) 
+            gtk_clist_set_column_title (GTK_CLIST (ctree), column, title->str);
 	
 	EDEBUG (("ctree-renderer", "Setting column %d to visible, and updating title", 
 		 column));
@@ -848,6 +878,38 @@ rendgtk_ctree_column_render (ENode * node)
     enode_attribs_sync (node);
 }
 
+static void
+rendgtk_ctree_column_parent (ENode * parent_node, ENode * child_node)
+{
+    GtkWidget *ctree;
+    GtkWidget *valign;
+    ENode *ctree_node;
+    gint column;
+
+    ctree_node = enode_parent (parent_node, "ctree");
+    if (!ctree_node) {
+	return;
+    } 
+
+    ctree = enode_get_kv (ctree_node, "bottom-widget");
+    if (!ctree)
+	return;
+
+    column = GPOINTER_TO_INT (enode_get_kv (parent_node, "ctree-column-number"));
+    
+    valign = enode_get_kv (parent_node, "bottom-widget");
+    if (!valign) {
+        valign = gtk_vbox_new (FALSE, 0);
+        gtk_clist_set_column_widget (GTK_CLIST (ctree), column, valign);
+        gtk_widget_show (valign);
+
+        enode_set_kv (parent_node, "bottom-widget", valign);
+    }
+
+    /* Now that we're sure we're all setup, just let the box pack handle it */
+    rendgtk_box_pack (parent_node, child_node);
+}
+
 static void
 rendgtk_ctree_column_destroy (ENode * node)
 {
@@ -1083,20 +1145,17 @@ rendgtk_ctree_render (ENode * node)
 
 
     gtk_signal_connect (GTK_OBJECT (ctree), "tree_select_row",
-			GTK_SIGNAL_FUNC (rendgtk_ctree_row_selected_callback),
-			node);
+			GTK_SIGNAL_FUNC (rendgtk_ctree_row_selected_callback), node);
     gtk_signal_connect (GTK_OBJECT (ctree), "tree_unselect_row",
-			GTK_SIGNAL_FUNC (rendgtk_ctree_row_unselected_callback),
-			node);
+			GTK_SIGNAL_FUNC (rendgtk_ctree_row_unselected_callback), node);
     gtk_signal_connect (GTK_OBJECT (ctree), "tree_expand",
-			GTK_SIGNAL_FUNC (rendgtk_ctree_row_expand_callback),
-			node);
+			GTK_SIGNAL_FUNC (rendgtk_ctree_row_expand_callback), node);
     gtk_signal_connect_after (GTK_OBJECT (ctree), "tree_collapse",
-			      GTK_SIGNAL_FUNC
-			      (rendgtk_ctree_row_collapse_callback), node);
+			      GTK_SIGNAL_FUNC (rendgtk_ctree_row_collapse_callback), node);
     gtk_signal_connect (GTK_OBJECT (ctree), "button_press_event",
-			GTK_SIGNAL_FUNC
-			(rendgtk_ctree_button_press_event_callback), node);
+			GTK_SIGNAL_FUNC (rendgtk_ctree_button_press_event_callback), node);
+    gtk_signal_connect (GTK_OBJECT (ctree), "click_column",
+                        GTK_SIGNAL_FUNC (rendgtk_ctree_column_clicked_callback), node);
 
     enode_set_kv (node, "top-widget", ctree);
     enode_set_kv (node, "bottom-widget", ctree);
@@ -1112,7 +1171,7 @@ rendgtk_ctree_render (ENode * node)
 	EDEBUG (("ctree-renderer", "Hiding column %i", i));
 	gtk_clist_set_column_visibility (GTK_CLIST (ctree), i, FALSE);
     }
-
+    
     enode_attribs_sync (node);
     rendgtk_show_cond (node, ctree);
 }
@@ -1252,6 +1311,7 @@ ctree_renderer_register (void)
     element->tag = "ctree-column";
     element->description = "Create columns for a clist";
     element->render_func = rendgtk_ctree_column_render;
+    element->parent_func = rendgtk_ctree_column_parent;
     element->destroy_func = rendgtk_ctree_column_destroy;
     element_register (element);
 
@@ -1264,8 +1324,7 @@ ctree_renderer_register (void)
 
     e_attr = g_new0 (ElementAttr, 1);
     e_attr->attribute = "active";
-    e_attr->description = "Specify if title should allow keypress events"
-	" etc., or just be a passive title.";
+    e_attr->description = "Specify if title should allow events, or just be a passive title.";
     e_attr->value_desc = "boolean";
     e_attr->set_attr_func = rendgtk_ctree_column_active_attr_set;
     element_register_attrib (element, e_attr);
