@@ -11,13 +11,27 @@ typedef struct {
     guint id;
 } button_press_timeout_t;
 
-/* DOUBLECLICK: second click must occur within 1/4 s * * order for double
- * click events * 1. GDK_BUTTON_PRESS * 2. GDK_BUTTON_RELEASE * 3.
- * GDK_BUTTON_PRESS * 4. GDK_2BUTTON_PRESS * 5. GDK_BUTTON_RELEASE * * *
- * TRIPLECLICK: second click must occur within 1/2 s * * order for triple
- * click events * 1. GDK_BUTTON_PRESS * 2. GDK_BUTTON_RELEASE * 3.
- * GDK_BUTTON_PRESS * 4. GDK_2BUTTON_PRESS * 5. GDK_BUTTON_RELEASE * 6.
- * GDK_BUTTON_PRESS * 7. GDK_3BUTTON_PRESS * 8. GDK_BUTTON_RELEASE */
+/* DOUBLECLICK: second click must occur within 1/4 s 
+ * order for doubled click events 
+ * 1. GDK_BUTTON_PRESS 
+ * 2. GDK_BUTTON_RELEASE 
+ * 3. GDK_BUTTON_PRESS 
+ * 4. GDK_2BUTTON_PRESS 
+ * 5. GDK_BUTTON_RELEASE 
+ * 
+ * 
+ *
+ * TRIPLECLICK: second click must occur within 1/2 s 
+ * order for triple click events 
+ * 1. GDK_BUTTON_PRESS 
+ * 2. GDK_BUTTON_RELEASE 
+ * 3. GDK_BUTTON_PRESS 
+ * 4. GDK_2BUTTON_PRESS 
+ * 5. GDK_BUTTON_RELEASE 
+ * 6. GDK_BUTTON_PRESS 
+ * 7. GDK_3BUTTON_PRESS 
+ * 8. GDK_BUTTON_RELEASE 
+ */
 
 static gint
 button_press_timeout_callback (gpointer callbackData)
@@ -50,6 +64,8 @@ button_press_event_callback (GtkWidget * widget,
     ENode *node = user_data;
     EBuf *multiclick;
 
+    EDEBUG (("clicks", "button_press_event_callback"));
+
     if (GDK_BUTTON_PRESS != event->type) {
 	return (TRUE);
     }
@@ -103,6 +119,8 @@ doubleclick_event_callback (GtkWidget * widget,
     button_press_timeout_t *data;
     EBuf *tripleclick;
 
+    EDEBUG (("clicks", "doubleclick_event_callback"));
+
     if (GDK_2BUTTON_PRESS != event->type) {
 	return (TRUE);
     }
@@ -142,6 +160,8 @@ tripleclick_event_callback (GtkWidget * widget,
     gchar *function;
     button_press_timeout_t *data;
 
+    EDEBUG (("clicks", "tripleclick_event_callback"));
+
     if (GDK_3BUTTON_PRESS != event->type) {
 	return (TRUE);
     }
@@ -741,6 +761,9 @@ rendgtk_widget_widget_event_set (ENode * node, EBuf * attr, EBuf * value)
 					   GTK_SIGNAL_FUNC
 					   (doubleclick_event_callback), node);
 	} else {
+	    gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
+				GTK_SIGNAL_FUNC (button_press_event_callback),
+				node);
 	    gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
 				GTK_SIGNAL_FUNC (doubleclick_event_callback),
 				node);
@@ -754,6 +777,12 @@ rendgtk_widget_widget_event_set (ENode * node, EBuf * attr, EBuf * value)
 					   GTK_SIGNAL_FUNC
 					   (tripleclick_event_callback), node);
 	} else {
+	    gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
+				GTK_SIGNAL_FUNC (button_press_event_callback),
+				node);
+	    gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
+				GTK_SIGNAL_FUNC (doubleclick_event_callback),
+				node);
 	    gtk_signal_connect (GTK_OBJECT (widget), "button_press_event",
 				GTK_SIGNAL_FUNC (tripleclick_event_callback),
 				node);
@@ -915,8 +944,8 @@ rendgtk_type_widget_attr_register (Element * element)
     e_attr->set_attr_func = rendgtk_widget_widget_font_set;
     element_register_attrib (element, e_attr);
 
-    /* Various signals to connect to arbitrary widgets * (only with windows
-     * though really..  may want to * check that in the future */
+    /* Various signals to connect to arbitrary widgets (only with windows though
+     * really..  may want to check that in the future */
     e_attr = g_new0 (ElementAttr, 1);
     e_attr->attribute = "onbuttonpress";
     e_attr->description = "Specify function called on a button press event.";
