@@ -6,7 +6,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <gtk/gtk.h>
 #include "entity.h"
 
 /* quiet compiler.. this is sometimes defined in perly.h */
@@ -65,6 +64,10 @@ XS (XS_Entity_enode_attrib_value_type);
 XS (XS_Entity_enode_attrib_possible_values);
 XS (XS_Entity_enode_attribs_sync);
 
+/* Key/value pair support */
+XS (XS_Entity_enode_get_kv);
+XS (XS_Entity_enode_set_kv);
+
 /* Node destruction */
 XS (XS_Entity_enode_destroy);
 XS (XS_Entity_enode_destroy_children);
@@ -118,14 +121,16 @@ initialize_perl (void)
 	perl_initialized = TRUE;
 
     perl_args[1] =
-	g_strconcat ("-I", g_get_home_dir (), "/entity/modules", NULL);
+	g_strconcat ("-I", g_get_home_dir (), "/.entity/perl-modules", NULL);
     perl_args[2] = g_strconcat ("-I", DATADIR, NULL);
     perl_args[3] = g_strconcat (DATADIR, "/", PERL_ENTITY_INIT, NULL);
     EDEBUG (("perl", "Calling: %s %s %s %s", perl_args[0], perl_args[1],
 	     perl_args[2], perl_args[3]));
 
     my_perl = perl_alloc ();
+    perl_init_i18nl10n (1);
     perl_construct (my_perl);
+    
     ret = perl_parse (my_perl, xs_init, 5, perl_args, NULL);
 
     if (ret) {
@@ -179,6 +184,10 @@ initialize_perl (void)
 	   XS_Entity_enode_attrib_possible_values, "Entity");
     newXS ("Entity::enode_attribs_sync", XS_Entity_enode_attribs_sync,
 	   "Entity");
+    
+    /* Key/value pair support */
+    newXS ("Entity::enode_get_kv", XS_Entity_enode_get_kv, "Entity");
+    newXS ("Entity::enode_set_kv", XS_Entity_enode_set_kv, "Entity");
 
     /* Node destruction */
     newXS ("Entity::enode_destroy", XS_Entity_enode_destroy, "Entity");
@@ -268,10 +277,10 @@ execute_perl_function (ENode * calling_node, gchar * function, GSList * args)
     EBuf *retbuf;
     static GString *perl_cmd = NULL;
     GSList *tmp;
-    GSList *nodelist = NULL;
+    GSList *perlargs = NULL;
     LangArg *arg;
     gchar *namespace;
-    SV *enode_sv = NULL;
+    SV *tmpsv = NULL;
     dSP;
     STRLEN n_a;
 
@@ -301,20 +310,19 @@ execute_perl_function (ENode * calling_node, gchar * function, GSList * args)
     ENTER;
     SAVETMPS;
     PUSHMARK (sp);
-
+	    
     tmp = args;
 
     while (tmp) {
 	arg = (LangArg *) tmp->data;
 
 	if (arg->type == LANG_NODE) {
-	    enode_sv = perl_get_enode (arg->data);
+	    tmpsv = perl_get_enode (arg->data);
 
-	    EDEBUG (("perl-embed", "SvROK is now %d, refcount %d",
-		     SvROK (enode_sv), SvREFCNT (enode_sv)));
-	    if (enode_sv) {
-		XPUSHs (enode_sv);
-		nodelist = g_slist_append (nodelist, enode_sv);
+	    EDEBUG (("perl-embed", "SvROK is now %d, refcount %d, making node %s",
+		     SvROK (tmpsv), SvREFCNT (tmpsv)));
+	    if (tmpsv) {
+		perlargs = g_slist_append (perlargs, tmpsv);
 	    } else {
 		EDEBUG (("perl-embed2", "Node not found :("));
 	    }
@@ -323,19 +331,29 @@ execute_perl_function (ENode * calling_node, gchar * function, GSList * args)
 		     arg->type == LANG_INT ||
 		     arg->type == LANG_DOUBLE || arg->type == LANG_BINSTRING) {
 	    if (arg->size > 0) {
-		EDEBUG (
-			("perl-embed", "arg = %s, size= %i", arg->data,
+		EDEBUG (("perl-embed", "arg = %s, size= %i", arg->data,
 			 arg->size));
-		XPUSHs (sv_2mortal (newSVpv (arg->data, arg->size)));
+                tmpsv = sv_2mortal (newSVpv (arg->data, arg->size));
 	    } else {
-		XPUSHs (sv_2mortal (newSVpv ("", 1)));
+		tmpsv = sv_2mortal (newSVpv ("", 0));
 	    }
+            /* The enode sv's are ref'd too.. this way we can just unref
+             * everything after the call */
+            SvREFCNT_inc (tmpsv);
+            perlargs = g_slist_append (perlargs, tmpsv);
+
 	}
 
 	enode_call_free_arg (arg);
 	tmp = tmp->next;
     }
 
+    tmp = perlargs;
+    while (tmp) {
+        XPUSHs (tmp->data);
+        tmp = tmp->next;
+    }
+    
     PUTBACK;
     stack_size = perl_call_pv (perl_cmd->str, G_EVAL);
     SPAGAIN;
@@ -354,11 +372,11 @@ execute_perl_function (ENode * calling_node, gchar * function, GSList * args)
     }
 
     /* Free our perl enodes. */
-    for (tmp = nodelist; tmp; tmp = tmp->next) {
-	enode_sv = tmp->data;
-	SvREFCNT_dec (enode_sv);
+    for (tmp = perlargs; tmp; tmp = tmp->next) {
+	tmpsv = tmp->data;
+	SvREFCNT_dec (tmpsv);
     }
-    g_slist_free (nodelist);
+    g_slist_free (perlargs);
 
     if (SvTRUE (ERRSV)) {
 	g_warning ("Perl: %s: %s", function, SvPV (ERRSV, n_a));
@@ -1212,6 +1230,61 @@ XS (XS_Entity_enode_attribs_sync)
     XSRETURN_EMPTY;
 }
 
+XS (XS_Entity_enode_get_kv)
+{
+    ENode *node;
+    char *key;
+    SV *value;
+    dXSARGS;
+    EARGS;
+
+    arg_warn (items == 2, "get_kv");
+    EARG_ENODE (node, 0);
+    if (node == NULL)
+	XSRETURN_EMPTY;
+    
+    EARG_STR (key, 1);
+    if (key == NULL)
+	XSRETURN_EMPTY;
+
+    CLEAR_STACK;
+
+    value = enode_get_kv (node, key);
+    if (!value) {
+        XSRETURN_UNDEF;
+    }
+    
+    XPUSHs (value);
+
+    XSRETURN (1);
+}
+
+XS (XS_Entity_enode_set_kv)
+{
+    ENode *node;
+    char *key;
+    SV *value;
+    dXSARGS;
+    EARGS;
+
+    arg_warn (items == 3, "set_kv");
+    EARG_ENODE (node, 0);
+    if (node == NULL)
+	XSRETURN_EMPTY;
+    
+    EARG_STR (key, 1);
+    if (key == NULL)
+	XSRETURN_EMPTY;
+
+    value = ST (2);
+    SvREFCNT_inc (value);
+        
+    /* Set value SV pointer */
+    enode_set_kv (node, g_strdup (key), value);
+
+    XSRETURN_EMPTY;
+}
+
 
 /* Node destruction */
 XS (XS_Entity_enode_destroy)
