@@ -68,6 +68,24 @@ py_enode_compare (PyEnodeObject * self, PyEnodeObject * other)
     return result;
 }
 
+static long
+py_enode_hash (PyEnodeObject * self)
+{
+  EBuf *path;
+  PyStringObject *string_path;
+  int string_hash;
+
+  ECHECK_RETVAL (self->enode !=NULL, 0);
+
+  path = enode_path (self->enode);
+  string_path = (PyStringObject*) PyString_FromStringAndSize (path->str,
+							      path->len);
+  string_hash = PyObject_Hash ((PyObject*)string_path);
+  Py_XDECREF (string_path);
+
+  return string_hash;
+}
+
 static int
 py_enode_mapping_length (PyEnodeObject * self)
 {
@@ -204,7 +222,7 @@ static PyTypeObject py_enode_type = {
     0,					/* tp_as_number */
     0,					/* tp_as_sequence */
     &py_enode_as_mapping,		/* tp_as_mapping */
-    (hashfunc) 0,			/* tp_hash */
+    (hashfunc) py_enode_hash,		/* tp_hash */
     (ternaryfunc) 0,			/* tp_call */
     (reprfunc) 0,			/* tp_str */
 
@@ -447,13 +465,11 @@ py_child (PyEnodeObject * self, PyObject * args)
 
     node = enode_child (self->enode, search);
     if (node == NULL) {
-	PyErr_SetString (PyExc_SystemError,
-			 "py_child: enode_child returned null.");
-	return NULL;
+	Py_INCREF (Py_None);
+	return Py_None;
     }
 
     py_node = py_enode_new_from_enode (node);
-
     return py_node;
 }
 
@@ -1519,6 +1535,11 @@ python_render (ENode * node)
     enode_call_reference_push (node);
 
     namespace = get_python_namespace ();
+    if (namespace == NULL) {
+      namespace = "__ENTITY_DEFAULT_NAMESPACE__";
+    }
+    
+    EDEBUG (("python-embed", "python_render: namespace is %s", namespace));
 
     code_pyobj = Py_CompileString (py_cmd->str, namespace, Py_file_input);
     if (code_pyobj == NULL) {
